/*
************************************************************************
*
*   GraphTransf.c - transformations of graphical display
*
*   Copyright (c) 1994
*
*   ETH Zuerich
*   Institut fuer Molekularbiologie und Biophysik
*   ETH-Hoenggerberg
*   CH-8093 Zuerich
*
*   SPECTROSPIN AG
*   Industriestr. 26
*   CH-8117 Faellanden
*
*   All Rights Reserved
*
*   Date of last modification : 95/03/10
*   Pathname of SCCS file     : /sgiext/molmol/src/graph/SCCS/s.GraphTransf.c
*   SCCS identification       : 1.2
*
************************************************************************
*/

#include <graph_transf.h>

#include <values.h>
#include <math.h>

#include <data_hand.h>

typedef struct {
  Mat4 rotM;
  Vec3 transV;
  BOOL transSet;
} RotData;

float
GraphDegToRad(float ang)
{
  return ang / 180.0 * M_PI;
}

static void
molRotate(DhMolP molP, void *clientData)
{
  RotData *dataP = clientData;
  Mat4 m, rotM;
  Vec3 transV;
  Vec4 v4;

  Mat4Copy(m, dataP->rotM);
  DhMolGetRotMat(molP, rotM);
  Mat4Mult(m, rotM);
  Mat4Ortho(m);
  DhMolSetRotMat(molP, m);

  /* adapt translation vectors so that all molecules rotate simultanously */
  if (dataP->transSet) {
    DhMolGetTransVect(molP, transV);
    Vec3Sub(transV, dataP->transV);
    Vec3To4(v4, transV);
    Mat4VecMult(v4, dataP->rotM);
    Vec4To3(transV, v4);
    Vec3Add(transV, dataP->transV);
    DhMolSetTransVect(molP, transV);
  } else {
    DhMolGetTransVect(molP, dataP->transV);
    dataP->transSet = TRUE;
  }
}

void
GraphRotate(Mat4 m)
{
  RotData data;

  Mat4Copy(data.rotM, m);
  data.transSet = FALSE;
  DhApplyMol(PropGetRef(PROP_MOVABLE, FALSE), molRotate, &data);
}

void
GraphRotateX(float ang)
{
  Mat4 m;
  float c = cosf(ang);
  float s = sinf(ang);

  Mat4Ident(m);
  m[1][1] = c;
  m[2][2] = c;
  m[1][2] = s;
  m[2][1] = -s;

  GraphRotate(m);
}

void
GraphRotateY(float ang)
{
  Mat4 m;
  float c = cosf(ang);
  float s = sinf(ang);

  Mat4Ident(m);
  m[0][0] = c;
  m[2][2] = c;
  m[0][2] = s;
  m[2][0] = -s;

  GraphRotate(m);
}

void
GraphRotateZ(float ang)
{
  Mat4 m;
  float c = cosf(ang);
  float s = sinf(ang);

  Mat4Ident(m);
  m[0][0] = c;
  m[1][1] = c;
  m[0][1] = s;
  m[1][0] = -s;

  GraphRotate(m);
}

static void
molTranslate(DhMolP molP, void *clientData)
{
  Vec3 transV;

  DhMolGetTransVect(molP, transV);
  Vec3Add(transV, clientData);
  DhMolSetTransVect(molP, transV);
}

void
GraphMove(Vec3 d)
{
  DhApplyMol(PropGetRef(PROP_MOVABLE, FALSE),  molTranslate, d);
}
